
local AuraModule = Dominos:NewModule('Debuff')
local AuraFrame = Dominos:CreateClass('Frame', Dominos.Frame)

function AuraModule:Load()
	self.frame = AuraFrame:New("debuff")
end

function AuraModule:Unload()
	self.frame:Free()
end

function AuraFrame:New(name)
	local f = AuraFrame.proto.New(self, name)
	f:SetFrameStrata('LOW')
	f:GetAuras()
--	f:LoadButtons()
	

	
	f:Layout()
	f:UpdateFilter()
	f:UpdateTarget()

	return f
end

function AuraFrame:GetDefaults()
	return {
		scale = 1,
		point = 'CENTER',
		y = 60,
		x = 0,
		spacing = 0,
		columns = 10,
		rows = 2,
		isRightToLeft = false,
		isBottomToTop = false,
		method = 1, --time, index or name
		direction = true,
		padding = 0,
	}
end

function AuraFrame:NumButtons()
	return self:NumColumns() * self:NumRows()
end

function AuraFrame:NumColumns()
	return self.sets.cols
end

function AuraFrame:NumRows()
	return self.sets.rows
end

function AuraFrame:ResetAuras()
	for i = 1, self.total do
		local aura = self.headerAura:GetAttribute("child" .. i)
		aura.tex:SetTexture("");
		aura.cd:Hide();
		aura.txt:Hide();
	end
end

function AuraFrame:GetAuras()
	self.headerAura = self.headerAura or CreateFrame("Frame", self:GetName().."DominosDebuffs", self, "SecureAuraHeaderTemplate")
	self.total = self.total or 0
	RegisterUnitWatch(self.headerAura)
	self.headerAura:SetScript("OnShow", function()
		self:UpdateAuras()
	end)
	self.headerAura:SetScript("OnHide", function()
		self:ResetAuras()
	end)
	self:SetScript("OnEvent", function(s, event, ...)
		self:OnEvent(event, ...)
	end)
	
	self.headerAura:SetAttribute("template", "DominosAuraTemplate"); -- must be the template name of your XML
	
	local headerAura = self.headerAura

	function headerAura:UpdateLayout(spacing, cols, rows, LR, TB, method, direction)
		for i,aura in self:GetParent():ActiveChildren() do
			if (tonumber(cols)and tonumber(rows)) and (tonumber(i) > (tonumber(cols)* tonumber(rows))) then
				aura:Hide()
			end
		end
		local base = (30 + spacing)
		headerAura:SetAttribute("minWidth", base);
		headerAura:SetAttribute("minHeight", base);

		headerAura:SetAttribute("wrapAfter", cols);
		headerAura:SetAttribute("maxWraps", rows);
		
		local hori, vert
		
		if not LR then	
			vert = "Left"
			headerAura:SetAttribute("xOffset", base);
			headerAura:SetAttribute("yOffset", 0);
		else
			vert = "Right"
			headerAura:SetAttribute("xOffset", -base);
			headerAura:SetAttribute("yOffset", 0);
		end
		if not TB then
			hori = "Top"
			headerAura:SetAttribute("wrapXOffset", 0);
			headerAura:SetAttribute("wrapYOffset", -base);	
		else
			hori = "Bottom"
			headerAura:SetAttribute("wrapXOffset", 0);
			headerAura:SetAttribute("wrapYOffset", base);
		end
		headerAura:SetAttribute("point", hori..vert);
		headerAura:SetAttribute("sortMethod", method); -- INDEX or NAME or TIME
		headerAura:SetAttribute("sortDirection", direction); -- - to reverse
	end
	
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("UNIT_TARGET");

	self:RegisterEvent("PLAYER_ENTERING_WORLD");	
	
	headerAura:UpdateLayout(3, "TopLeft", 4, 4, true, true,"Time", "+")
end

local function siter_active_children(header, i)
	i = i + 1;
	local child = header:GetAttribute("child" .. i);
	if child and child:IsShown() then
		return i, child, child:GetAttribute("index");
	end
end

function AuraFrame:ActiveChildren()
	return siter_active_children, self.headerAura, 0
end

function AuraFrame:UpdateAuras()
	self:ResetAuras()
	self.total = 0
	for i,aura in self:ActiveChildren() do
		self.total = self.total + 1
		
		local aura = self.headerAura:GetAttribute("child" .. i)
		
		local name, _, icon, count, dedebuffType, duration, expirationTime = UnitAura(self.headerAura:GetAttribute("unit"), aura:GetID(), self.headerAura:GetAttribute("filter"));
		if name then
			aura.tex:SetTexture(icon);
			aura.tex:SetTexCoord(.15,.85,.15,.85)
			aura.cd:SetCooldown(expirationTime - duration, duration);
			if not (count>1)then count=""end
			aura.txt:SetText(count);
			aura.cd:Show();
		else
			aura.tex:SetTexture("");
			aura.cd:Hide();
			aura.txt:SetText("");
		end
	end
end

function AuraFrame:OnEvent(event, ...)
	self:UpdateAuras()
end

function AuraFrame:GetFilter()
	return "HARMFUL"
end

function AuraFrame:UpdateFilter()
	self.headerAura:SetAttribute("filter", self:GetFilter()); -- to activate UNITAURA event refresh
	self:GetScript("OnEvent")()
end

function AuraFrame:GetTarget()
	return self.sets.target or "player"
end

function AuraFrame:UpdateTarget()
	self.headerAura:SetAttribute("unit", self:GetTarget())
	self:GetScript("OnEvent")()
end

function AuraFrame:GetSpacing()

	return self.sets.spacing
end

function AuraFrame:SetTarget(unit)
	self.sets.target = unit or "player"
	self:UpdateTarget()
end

local items = {"Time", "Index", "Name",}

function AuraFrame:Layout()
	if not InCombatLockdown() then
		local sets = self.sets
		local w,h = 30, 30
		local newWidth = max((((30 + sets.spacing) * sets.columns) - sets.spacing) +(sets.padding*2), 8)
		local newHeight = max((((30 + sets.spacing) * sets.rows) - sets.spacing) +(sets.padding*2), 8)
		self:SetSize(newWidth, newHeight)

		local hori, vert, padhori, padvert
		
		if not sets.isRightToLeft then	
			hori = "Left"
			padhori = sets.padding
		else
			hori = "Right"
			padhori = -sets.padding
		end
		if not sets.isBottomToTop then
			vert = "Top"
			padvert = -sets.padding
		else
			vert = "Bottom"
			padvert = sets.padding
		end

		self.headerAura:ClearAllPoints()
		self.headerAura:SetPoint(vert..hori, self, vert..hori, padhori, padvert)
		local dir = "+"
		if sets.direction == 1 then
			dir = "-"
		end

		local method = items[sets.method]
		self.headerAura:UpdateLayout(sets.spacing, sets.columns, sets.rows, sets.isRightToLeft, sets.isBottomToTop, method, dir)
		self:GetScript("OnEvent")()
	end
end

function AuraFrame:SetSpacing(spacing)
    self.sets.spacing = spacing
    self:Layout()
end

local function NewSlider(panel, name, min, max, arg)
        local slider =  panel:NewSlider({
            name = name,
            min = min,
            max = max,
            get = function(self) --Getter
                return panel.owner.sets[arg]
            end,
            set = function(self) --Setter
                local owner = panel.owner
                owner.sets[arg] = self:GetValue()
                owner:Layout()
                return owner.sets[arg]
            end,
        })
end

local function NewCheckButton(panel, name, arg)
        local c = panel:NewCheckButton{
            name = name,
            get = function() return panel.owner.sets[arg] end,
            set = function(self, enable)
                panel.owner.sets[arg] = self:GetChecked()
                panel.owner:Layout()
            end
        }
end

local function NewDropdown(panel, name, arg, items)
        local c = panel:NewDropdown{
		name = name,

		get = function()
			return panel.owner.sets[arg]
		end,

		set = function(_, value)
		    panel.owner.sets[arg] = value
		    panel.owner:Layout()
		end,

		items = items
	}
end

local function AddLayoutPanel(menu)
	local p = menu:NewPanel("Layout")
	p:NewScaleSlider()
    NewSlider(p, "Padding", -13, 32, "padding")
    NewSlider(p, "Columns", 1, 20, "columns")
    NewSlider(p, "Rows", 1, 20, "rows")
    p:NewSpacingSlider()
	p:NewFadeSlider()
	p:NewOpacitySlider()
end



local function AddSortPanel(self)
	local p = self:NewPanel("Sorting")

    NewCheckButton(p, "Reverse", "direction")
    NewDropdown(p, "Sort Method", "method", {
			{text = "Time", value = '1'},
			{text = "Index", value = '2'},
			{text = "Name", value = '3'},
		})

	return p
end


--viable unit check
local base = {
	"player",
	"target",
	"pet",
	"focus",
	"arena%d+",
	"boss%d+",
	"party%d+",
	"partypet%d+",
	"raid%d+",
	"raidpet%d+",
	"mouseover",
}

local max = {
	raid = 40,
	raidpet = 40,
	arena = 4,
	boss = 4,
	party = 4,
	partypet = 4,
}

local function IsViable(unit)
	local keep = unit
	unit = string.lower(unit)
	local count = 0
	if string.match (unit, "target" , 2) then
		_, count = string.gsub(string.sub (unit, 2, string.len(unit)), "target", "")
		if (unit == "player") and count > 0 then
			return nil
		else
			unit = string.gsub (unit, "target", "", count)
		end
	end	
	local can
	for test, match in pairs(base) do
		if string.match(string.gsub(match, "%d+", ""), string.gsub(unit, "%d+", "") ) then
			hasLimit = max[string.gsub( unit, "%d+", "" ) ]
			if hasLimit then
				local o = string.gsub(unit, string.gsub(unit, "%d+", ""),"")
				o = tonumber(o)
				if o and ( (o < 1) or (o>hasLimit) ) then
					return nil
				end
			end
			return string.lower(keep)
		end
	end
end

local function AddAdvancedOptions(menu)
    local L = LibStub('AceLocale-3.0'):GetLocale('Dominos-Config')

    local panel = menu:NewPanel(L.Advanced)
	panel:NewClickThroughCheckbox()
	panel:NewShowInOverrideUICheckbox()
	panel:NewShowInPetBattleUICheckbox()
	panel.showStatesEditBox = panel:NewTextInput{
		name = L.ShowStates,
		multiline = true,
		width = 290,
		height = 64,
		get = function() return panel.owner:GetShowStates() end,
		set = function(_, value) panel.owner:SetShowStates(value) end
	}

	panel.TargetingEditBox = panel:NewTextInput{
		name = "Target",
		multiline = true,
		width = 290,
		height = 64,
		get = function() return panel.owner:GetTarget() or '' end,
		set = function(self, value)
            local text = value
            if IsViable(text) then
                panel.owner:SetTarget(text ~= '' and text or nil)
            else
                print("Invalid Unit: "..text)
            end
            panel.TargetingEditBox.editBox:ClearFocus()
        end
	}

end

function AuraFrame:CreateMenu()
	local menu = Dominos:NewMenu(self.id)
	AddLayoutPanel(menu)
	AddSortPanel(menu)
	AddAdvancedOptions(menu)
	self.menu = menu
end